/* vi:set cindent: */
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "getopt.h"
#include <stdarg.h>
#include <sys/types.h>
#include <sys/wait.h>

int debug;

int
sockprintf (int socket, char *format,...)
{
  va_list ap;
  char buf[10240];

  va_start (ap, format);
  vsprintf (buf, format, ap);
  va_end (ap);
  return write (socket, buf, strlen (buf));
}

char *packnumber(char *number)
{
    char *s1,*s2;
    
    s1 = s2 = number;
    while (*s1 != 0) {
	 if (*s1 != '-')
	     *s2++ = *s1;
	 s1++;
    }
    *s2 = 0;
    return number;
}

struct network {
    char *prefix;
    char *program;
    struct network *next;
};

struct network *networks;

int load_networks()
{
    FILE *f;
    char line[0x100];
    int i;
    char *prefix, *program;
    int lineno;
    struct network *net;
    
    f = fopen("/etc/sms.networks", "r");
    if (f == NULL) {
	perror("/etc/sms.networks");
	exit(-1);
    }
    lineno = 0;
    
    while (fgets(line,0x100,f)) {
	lineno++;
	if (line[strlen(line)-1] == '\n')
	    line[strlen(line)-1] = 0;
	/* remove all # */
	for (i=0;i<strlen(line);i++)
	    if (line[i] == '#')
		line[i] = 0;
	
	while ( (strlen(line) > 1) && 
		(line[strlen(line)-1] == ' ' || line[strlen(line)-1] == '\t'))
	    line[strlen(line)-1] = 0;
	
	if (line[0] == 0)
	    continue;
	
	/* parse */
	prefix = line;
	
	program = prefix;
	while (*program != ' ' && *program != '\t' && *program)
	    program++;
	
	if (*program == 0) {
	    fprintf(stderr, "Invalid line %u in /etc/sms.networks\n", lineno);
	    exit(1);
	}
	*program++ = 0;
	while (*program == ' ' || *program == '\t')
	    program++;
	
	net = (struct network*)malloc(sizeof(struct network));
	if (!net) {
	    fprintf(stderr, "Not enough memory\n");
	    exit(2);
	}
	
	net->prefix = strdup(prefix);
	if (!net->prefix) {
	    free(net);
	    fprintf(stderr, "Not enough memory\n");
	    exit(2);
	}

	net->program = strdup(program);
	if (!net->program) {
	    free(net);
	    free(net->prefix);
	    fprintf(stderr, "Not enough memory\n");
	    exit(2);
	}
	
	net->next = networks;
	networks = net;
    }
    fclose(f);
    return 1;
}


int
smssend (char *number, char *email, unsigned char *message, char *myphone)
{
    int pid;
    struct network *net;
    int status;			/* 0=no prog, 1=ok, -1=error from child */
    char *argv[40];
    int argc;
    int fds[2];
    int pstat;

    status = 0;

    packnumber(number);
    net = networks;
    while (net) {
	if (!strncmp(number, net->prefix, strlen(net->prefix))) {
	    argc = 0;
	    argv[argc++] = net->program;
	    argv[argc++] = number;
	    if (debug)
		    argv[argc++] = "-d";
	    if (email) {
		argv[argc++] = "-n";
		argv[argc++] = email;
	    }
	    if (myphone) {
		argv[argc++] = "-x";
		argv[argc++] = myphone;
	    }

	    argv[argc] = 0;
	    
	    pipe(fds);
	    pid = fork();
	    if (pid == 0) {
		close(fds[0]);
		close(STDIN_FILENO);
		dup2(fds[1], STDIN_FILENO);
		close(fds[1]);
		printf("Executing %s\n", net->program);
	        execv(net->program, argv);
		_exit(127);
	    }
	    close(fds[1]);
	    write(fds[0], message, strlen(message));
//	    write(fds[0], "\n", 1);
	    close(fds[0]);

	    waitpid(pid, &status, 0);
	    if (!WIFEXITED(status))
		status = -1;
	    if (WEXITSTATUS(status)!=0)
		status = -1;
	       else status = 1;
	    if (WIFSIGNALED(status))
		status = -1;
	}
	if (status == 1)
	    return 0;
	net = net->next;
    }
    
    if (status == 0) 
	fprintf(stderr, "No SMS program defined for number %s\n", number);
    if (status < 0)
	fprintf(stderr, "SMS program returned error.\n");
    return -1;
}

void usage(char *progname)
{
    printf("Usage: %s [-d] [-l] [-n email@address] [-x phonenum] [phone number(s)]\n"
	   "       %s -V\n"
	   "       %s -h\n"
	   "\n"
	   "Options:\n"
	   "    -d            Debug mode\n"
	   "    -h            Show this help screen\n"
	   "    -l            Line mode, for use with NNTP (not implemented)\n",
	   "    -n email      Send delivery notification via e-mail\n"
	   "    -V            Display version\n",
	   progname, progname, progname);
}

int main(int argc, char **argv)
{
    char *email;
    char *message;
    char line[0x100];
    char *phoneno;
    char *myphone;
    int havephone;
    char *progname;
    char ch;
    int errs;
    
    progname = argv[0];
    debug = 0;
    networks = NULL;
    myphone = NULL;
    
    while ((ch = getopt(argc, argv, "Vhn:dx:"))!=-1) {
	switch(ch) {
	    case 'V':
		printf("Version 0.1 (compiled on " __DATE__ ")\n");
		break;
	    case 'h':
		usage(progname);
		exit(0);
 	    case 'n':
		email = optarg;
		break;
 	    case 'x':
		myphone = optarg;
		break;
	    case 'l':
		break;
            case 'd':
		debug = 1;
		break;
	    default:
		fprintf(stderr,"Invalid option %c, try %s -h\n", ch, progname);
		exit(1);
	}
    }
    
    load_networks();
    
    argc -= optind;
    argv += optind;
	
    havephone = argc!=0;

    email = 0;
    
    message = (char *)malloc(1024);
   
    if (!havephone) {
      printf("Enter phone number: ");
      phoneno = (char *)malloc(1024);
      fgets(phoneno, 1024, stdin);
      if (phoneno[strlen(phoneno)-1] == '\n')
	  phoneno[strlen(phoneno)-1] = 0;
    }

    if (isatty(STDIN_FILENO)) 
	printf("Enter your message, end with '.':\n");

    message[0] = 0;    
    while (1) {
	if (isatty(STDIN_FILENO)) {
           printf("> ");
	   fflush(stdout);
        }
	if (fgets(line,0x100,stdin) == NULL)
	    break;
	if (line[strlen(line)-1] == '\n')
	    line[strlen(line)-1] = 0;
	if (!strcmp(line,"."))
	    break;
	if (message[0] && strlen(message)<1023)
	   strcat(message," ");
	if (strlen(message)+strlen(line)<1024)
		strcat(message,line);
	else {
	    memcpy(message, line, 1023-strlen(message));
	    message[1023] = 0;
	    printf("Warning: Message too long, truncated\n");
	    break;
	}
    }
 
    errs = 0;   
    if (havephone) {
	while(argc--) {
            if ( smssend(argv[0], email, message, myphone) < 0)
		errs++;
	    argv++;
	}
    }else{
	if ( smssend(phoneno, email, message, myphone) < 0)
	     errs++;
    }
    
    if (errs) {
	fprintf(stderr, "Error(s), some messages were not send!\n");
	exit(1);
    }else {
	printf("Ok, all messages sent to SMS center.\n");
	exit(0);
    }
    
    return 0;
}
